@@ -1,18 +1,23 @@
-pub mod class;
+//! eJNI
+
+#![deny(clippy::missing_safety_doc)]
+#![warn(missing_docs)]
+
+mod class;
 pub use class::*;
 
-pub mod object;
+mod object;
 pub use object::*;
 
-pub mod primitives;
+mod primitives;
 pub use primitives::*;
 
-pub mod abstractions;
+mod abstractions;
 pub use abstractions::*;
 
 #[cfg(test)]
 mod test {
-    use jni::{JavaVM, JNIVersion, InitArgsBuilder};
+    use jni::{InitArgsBuilder, JNIVersion, JavaVM};
     use lazy_static::lazy_static;
     use std::sync::Mutex;
     lazy_static! {
@@ -26,4 +31,4 @@ mod test {
             Mutex::new(JavaVM::new(jvm_args).unwrap())
         };
     }
-}
\ No newline at end of file
+}
